Package: TFBSTools
Version: 1.50.0
Date: 2025-07-22
Title: Software Package for Transcription Factor Binding Site (TFBS)
        Analysis
Description: TFBSTools is a package for the analysis and manipulation
        of transcription factor binding sites. It includes matrices
        conversion between Position Frequency Matirx (PFM), Position
        Weight Matirx (PWM) and Information Content Matrix (ICM). It
        can also scan putative TFBS from sequence/alignment, query
        JASPAR database and provides a wrapper of de novo motif
        discovery software.
Authors@R: person("Ge", "Tan", email="ge_tan@live.com",
                  role = c("aut", "cre"))
VignetteBuilder: knitr
Imports: Biobase(>= 2.28), Biostrings(>= 2.36.4), pwalign,
        BiocGenerics(>= 0.14.0), BiocParallel(>= 1.2.21), BSgenome(>=
        1.36.3), caTools(>= 1.17.1), DirichletMultinomial(>= 1.10.0),
        Seqinfo, GenomicRanges(>= 1.20.6), gtools(>= 3.5.0), grid,
        IRanges(>= 2.2.7), methods, DBI (>= 0.6), RSQLite(>= 1.0.0),
        rtracklayer(>= 1.28.10), seqLogo(>= 1.34.0), S4Vectors(>=
        0.9.25), TFMPvalue(>= 0.0.5), XML(>= 3.98-1.3), XVector(>=
        0.8.0), parallel
Depends: R (>= 3.2.2)
Suggests: BiocStyle(>= 1.7.7), JASPAR2014(>= 1.4.0), knitr(>= 1.11),
        testthat, JASPAR2016(>= 1.0.0), JASPAR2018(>= 1.0.0), rmarkdown
License: GPL-2
URL: https://github.com/ge11232002/TFBSTools
BugReports: https://github.com/ge11232002/TFBSTools/issues
Type: Package
biocViews: MotifAnnotation, GeneRegulation, MotifDiscovery,
        Transcription, Alignment
NeedsCompilation: yes
LazyData: yes
Collate: AllGenerics.r AllClasses.r show-methods.r util-methods.r
        XMatrix-methods.r XMatrixList-methods.r SiteSet-methods.r
        coercion-methods.r DB-methods.r JASPAR.R
        PairwiseAlignment-methods.r ICM-methods.r PWM-methods.r
        Motif-methods.r PFM-methods.r Wrappers-methods.r
        DirichletMixture.r TFFM.R TFFMseqLogo.R IO.R Enrichment.R
Config/pak/sysreqs: make libgsl0-dev libbz2-dev liblzma-dev libxml2-dev
        libssl-dev xz-utils zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:38:20 UTC
RemoteUrl: https://github.com/bioc/TFBSTools
RemoteRef: RELEASE_3_23
RemoteSha: 45fd53825d6528101f16cc09dee94df57c38274a
Packaged: 2026-04-29 13:52:39 UTC; root
Author: Ge Tan [aut, cre]
Maintainer: Ge Tan <ge_tan@live.com>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 14:09:33 UTC; unix
